@@ -929,6 +929,8 @@ set_prolog_flag_unlocked(DECL_LD Module m, atom_t k, term_t value, int flags)
 	  return FALSE;
       } else if ( k == ATOM_string_stack_tripwire )
       { LD->fli.string_buffers.tripwire = (unsigned int)i;
+      } else if ( k == ATOM_heartbeat )
+      { LD->yield.frequency = i/16;
       }
       break;
     }
@@ -1441,7 +1443,6 @@ initPrologFlagTable(void)
   }
 }
 
-
 void
 initPrologFlags(void)
 { GET_LD
@@ -1460,6 +1461,7 @@ initPrologFlags(void)
 #endif
   setPrologFlag("file_name_case_handling", FT_ATOM,
 		stringAtom(currentFileNameCaseHandling()));
+  setPrologFlag("path_max", FT_INTEGER|FF_READONLY, PATH_MAX);
   setPrologFlag("version", FT_INTEGER|FF_READONLY, PLVERSION);
   setPrologFlag("dialect", FT_ATOM|FF_READONLY, "swi");
   if ( systemDefaults.home )
@@ -1483,14 +1485,6 @@ initPrologFlags(void)
   setPrologFlag("vmi_builtin", FT_BOOL, TRUE, PLFLAG_VMI_BUILTIN);
   setPrologFlag("warn_override_implicit_import", FT_BOOL, TRUE,
 		PLFLAG_WARN_OVERRIDE_IMPLICIT_IMPORT);
-  setPrologFlag("c_cc",	     FT_ATOM, C_CC);
-  setPrologFlag("c_libs",    FT_ATOM, C_LIBS);
-#ifdef C_LIBDIR
-  setPrologFlag("c_libdir",  FT_ATOM, C_LIBDIR);
-#endif
-  setPrologFlag("c_libplso", FT_ATOM, C_LIBPLSO);
-  setPrologFlag("c_ldflags", FT_ATOM, C_LDFLAGS);
-  setPrologFlag("c_cflags",  FT_ATOM, C_CFLAGS);
   setPrologFlag("tmp_dir", FT_ATOM, SWIPL_TMP_DIR);
 #if defined(O_LARGEFILES) || SIZEOF_LONG == 8
   setPrologFlag("large_files", FT_BOOL|FF_READONLY, TRUE, 0);
@@ -1507,10 +1501,18 @@ initPrologFlags(void)
   setPrologFlag("shared_table_space", FT_INTEGER, GD->options.sharedTableSpace);
 #endif
   setPrologFlag("stack_limit", FT_INTEGER, LD->stacks.limit);
-#if defined(HAVE_DLOPEN) || defined(HAVE_SHL_LOAD) || defined(EMULATE_DLOPEN)
-  setPrologFlag("open_shared_object",	  FT_BOOL|FF_READONLY, TRUE, 0);
-  setPrologFlag("shared_object_extension",	  FT_ATOM|FF_READONLY, SO_EXT);
+#ifdef O_DYNAMIC_EXTENSIONS
+  setPrologFlag("open_shared_object",	     FT_BOOL|FF_READONLY, TRUE, 0);
+  setPrologFlag("shared_object_extension",   FT_ATOM|FF_READONLY, SO_EXT);
   setPrologFlag("shared_object_search_path", FT_ATOM|FF_READONLY, SO_PATH);
+  setPrologFlag("c_cc",	     FT_ATOM, C_CC);
+  setPrologFlag("c_libs",    FT_ATOM, C_LIBS);
+#ifdef C_LIBDIR
+  setPrologFlag("c_libdir",  FT_ATOM, C_LIBDIR);
+#endif
+  setPrologFlag("c_libplso", FT_ATOM, C_LIBPLSO);
+  setPrologFlag("c_ldflags", FT_ATOM, C_LDFLAGS);
+  setPrologFlag("c_cflags",  FT_ATOM, C_CFLAGS);
 #endif
   setPrologFlag("address_bits", FT_INTEGER|FF_READONLY, sizeof(void*)*8);
 #ifdef HAVE_POPEN
@@ -1644,6 +1646,7 @@ initPrologFlags(void)
   setPrologFlag("signals", FT_BOOL|FF_READONLY,
 		truePrologFlag(PLFLAG_SIGNALS), PLFLAG_SIGNALS);
   setPrologFlag("packs", FT_BOOL, GD->cmdline.packs, 0);
+  setPrologFlag("heartbeat", FT_INTEGER, 0);
 
 #if defined(__WINDOWS__) && defined(_DEBUG)
   setPrologFlag("kernel_compile_mode", FT_ATOM|FF_READONLY, "debug");
