@@ -515,9 +515,7 @@ dispatch_signal(int sig, int sync)
   }
 
   if ( (LD->critical || (true(sh, PLSIG_SYNC) && !sync))
-#if O_SIGNALS
-       && sig != SIGINT
-#endif
+       && sh->handler != PL_interrupt
        && !is_fatal_signal(sig)	)
   { PL_raise(sig);			/* wait for better times! */
     return;
@@ -1405,7 +1403,7 @@ PRED_IMPL("$on_signal", 4, on_signal, 0)
     } else if ( a == ATOM_debug )
     { sh = prepareSignal(sign, 0);
 
-      sh->handler = (handler_t)PL_interrupt;
+      sh->handler = PL_interrupt;
       sh->predicate = NULL;
 
     } else
@@ -1796,6 +1794,7 @@ freePrologLocalData(PL_local_data_t *ld)
   }
 
   cleanAbortHooks(ld);
+  unreferenceStandardStreams(ld);
 }
 
 /* The following definitions aren't necessary for compiling, and in fact
